#!/usr/bin/env node
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nodeElmCompiler = require('node-elm-compiler');

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var filename = (process.argv[2] || '').trim();

if (!filename || filename === '-h' || filename === '--help') {
  console.log('Usage: run-elm [FILENAME]');
  process.exit(1);
}

if (!_shelljs2.default.which('elm')) {
  console.error('Error: No elm global binary available. Please install with `npm install -g elm`.');
  process.exit(1);
}

function promisify(fn) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return new Promise(function (resolve, reject) {
      fn.apply(undefined, args.concat([function (err, result) {
        if (err) {
          reject(err);
        } else {
          resolve(result);
        }
      }]));
    });
  };
}

var stat = promisify(_fs2.default.stat);
var readFile = promisify(_fs2.default.readFile);
var writeFile = promisify(_fs2.default.writeFile);
var unlink = promisify(_fs2.default.unlink);

var runModuleName = 'RunElmMain';
var templatePath = _path2.default.resolve(__dirname, runModuleName + '.elm.template');

var moduleName = _path2.default.basename(filename, '.elm');
var outputDir = _path2.default.resolve(_path2.default.dirname(filename));
var outputElmFilename = _path2.default.join(outputDir, runModuleName + '.elm');
var outputCompiledFilename = _path2.default.join(outputDir, 'run-elm-main.js');

function cleanup() {
  return Promise.all([unlink(outputElmFilename).catch(function () {}), unlink(outputCompiledFilename).catch(function () {})]);
}

function getTemplate() {
  return readFile(templatePath, 'utf-8');
}

function createElmFile(template) {
  var contents = template.replace('{module}', moduleName);
  return writeFile(outputElmFilename, contents, 'utf-8');
}

function compileElmFile() {
  _shelljs2.default.cd(outputDir);

  return (0, _nodeElmCompiler.compileToString)([outputElmFilename], { yes: true }).then(function (contents) {
    return writeFile(outputCompiledFilename, contents);
  });
}

function runElmFile() {
  return new Promise(function (resolve) {
    var worker = require(outputCompiledFilename)[runModuleName].worker;

    worker().ports.sendOutput.subscribe(function (output) {
      console.log(output);
      resolve();
    });
  });
}

function handleError(err) {
  var message = void 0;

  if ((typeof err === 'undefined' ? 'undefined' : _typeof(err)) === 'object' && 'message' in err) {
    if (/does not expose `output`/.test(err.message)) {
      message = 'Elm module `' + moduleName + '` does not define a String `output` variable.';
    } else {
      message = err.message;
    }
  } else {
    message = err;
  }

  console.error('Error:', message);
}

stat(filename).then(getTemplate, function () {
  return Promise.reject('Elm file \'' + filename + '\' does not exist');
}).then(createElmFile).then(compileElmFile).then(runElmFile).then(cleanup).catch(function (err) {
  cleanup().then(function () {
    handleError(err);
    process.exit(1);
  });
});